/// <summary>
///ESChannelSample.cs
///
///This sample demonstrates adding an EventServiceChannel to the EventService.
///
///Requirement:
///
///The ChannelledEvent.mon must be injected to the correlator before running this 
///example.  The ChannelledEvent.mon simply re-sends the ChannelledEvent() event by
///concatenating the string  "(echoed)" to the end of the member.
///
///In order to check the SendEvent() actually works, bring up the engine_receive
///to keep track of the event  ChannelledEvent("hi there (channelled)")   echoed back
///by the monitorscript.
///
///$Copyright(c) 2010 Progress Software Corporation. All Rights Reserved.$ 
///$Copyright (c) 2013-2015 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
///Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
///
/// </summary>

using System;
using System.Threading;

using Apama;
using Apama.Event;
using Apama.Event.Parser;
using Apama.Services.Event;
using Apama.Util;

public class ESChannelSample {
	private static readonly Logger logger = Logger.GetLogger(typeof(ESChannelSample));

	volatile bool received = false;

	/// <summary>
	///Simply sends the ChannelledEvent("hi there")
	///event to the correlator and process the response.
	///
	/// </summary>
	public void RunSample(string host, int port) 
	{
		// get a IEventService instance from the EventServiceFactory
		using (IEventService eventService = EventServiceFactory.CreateEventService(host, port, "ESChannelSample"))
		{
			// Add the channel to those monitored by the event service
			IEventServiceChannel ourChannel = eventService.AddChannel(
					"eventService.sample.channel", null);
	
			// Create an EventType to describe the event we are expecting to receive
			EventType channelEventType = new EventType("ChannelledEvent",
					new Field[] { new Field("s", StringFieldType.Type) });
	
			// Add a listener that will be notified whenever an event of the
			// specified type is received on the channel
			ourChannel.AddEventListener(new MyEventListener(this), channelEventType);
	
			// Send an event to the correlator
			try 
			{
				Event cEvent = new Event("ChannelledEvent(\"hi there\")");
				eventService.SendEvent(cEvent);
			} 
			catch (EngineException e) 
			{
				logger.Error("Caught EngineException: ", e);
			}
	
			/// <summary>
			///Since this is a console application, we need to wait for the callback to acknowledge
			///the event response on the channel.  Otherwise this application will just terminate
			///before receiving the response.
			/// </summary>
			while (!received) 
			{
				try 
				{
					Thread.CurrentThread.Join(500);
				} 
				catch (ThreadInterruptedException e) 
				{
					logger.Info("Interrupted while waiting: ", e);
				}
			}
		} // using() will destroy the event service after using it

	}

	/// <summary>
	///Inner class to implement our callback method that is notified when events
	///are received.
	/// </summary>
	class MyEventListener : EventListenerAdapter 
	{
		private ESChannelSample enclosingInstance;
		
		public MyEventListener(ESChannelSample e)
		{
			enclosingInstance = e;
		}

		/// <summary>
		///Callback method that is called when a new Event is received.
		///
		/// </summary>
		///<param name="evt">The received Event.</param>
		public override void HandleEvent(Event evt) 
		{
			Console.WriteLine("MyEventListener.HandleEvent() called, event = "
					+ evt);
			// acknowledge the event response
			enclosingInstance.received = true;
		}
	}

	public static void Main(string[] args) 
	{
		if ((args.Length == 2) && (int.Parse(args[1]) > 0))
		{
			string host = args[0];
			int port = int.Parse(args[1]);
			new ESChannelSample().RunSample(host, port);
		}
		else
		{
			Console.WriteLine("Usage: ESChannelSample.exe <host> <port>");
		}
	}
}
